/**
 * Copyright (c) 2007, 2020 Borland Software Corporation, CEA LIST, Artal and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *    Dmitry Stadnik (Borland) - initial API and implementation
 *    Michael Golubev (Montages) - #386838 - migrate to Xtend2
 *    Aurelien Didier (ARTAL) - aurelien.didier51@gmail.com - Bug 569174
 */
package xpt.editor;

import com.google.common.base.Objects;
import com.google.inject.Inject;
import com.google.inject.Singleton;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenApplication;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenDiagram;
import org.eclipse.papyrus.gmf.codegen.xtend.annotations.Localization;
import org.eclipse.papyrus.gmf.codegen.xtend.annotations.MetaDef;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import plugin.Activator;
import xpt.Common;
import xpt.Externalizer;
import xpt.ExternalizerUtils_qvto;
import xpt.diagram.commands.CreateShortcutDecorationsCommand;

@Singleton
@SuppressWarnings("all")
public class ShortcutCreationWizard {
  @Inject
  @Extension
  private Common _common;
  
  @Inject
  @Extension
  private ExternalizerUtils_qvto _externalizerUtils_qvto;
  
  @Inject
  private Externalizer xptExternalizer;
  
  @Inject
  private Activator xptActivator;
  
  @Inject
  private ModelElementSelectionPage xptModelElementSelectionPage;
  
  @Inject
  private CreateShortcutDecorationsCommand xptCreateShortcutDecorationsCommand;
  
  @MetaDef
  public CharSequence className(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("ShortcutCreationWizard");
    return _builder;
  }
  
  public CharSequence packageName(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    String _packageName = it.getEditorGen().getEditor().getPackageName();
    _builder.append(_packageName);
    return _builder;
  }
  
  public CharSequence qualifiedClassName(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _packageName = this.packageName(it);
    _builder.append(_packageName);
    _builder.append(".");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    return _builder;
  }
  
  public CharSequence fullPath(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _qualifiedClassName = this.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    return _builder;
  }
  
  public CharSequence ShortcutCreationWizard(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _copyright = this._common.copyright(it.getEditorGen());
    _builder.append(_copyright);
    _builder.newLineIfNotEmpty();
    _builder.append("package ");
    CharSequence _packageName = this.packageName(it);
    _builder.append(_packageName);
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _generatedClassComment = this._common.generatedClassComment("Allows to select foreign model element and add shortcut to the diagram.");
    _builder.append(_generatedClassComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    _builder.append(" extends org.eclipse.jface.wizard.Wizard {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t ");
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment, "\t ");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private ReferencedElementSelectionPage referencedElementSelectionPage;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_1 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_1, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private org.eclipse.emf.transaction.TransactionalEditingDomain editingDomain;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_2 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_2, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public ");
    CharSequence _className_1 = this.className(it);
    _builder.append(_className_1, "\t");
    _builder.append("(org.eclipse.emf.ecore.EObject modelElement, org.eclipse.gmf.runtime.notation.View view,");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t");
    _builder.append("org.eclipse.emf.transaction.TransactionalEditingDomain editingDomain) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("referencedElementSelectionPage = new ReferencedElementSelectionPage(");
    CharSequence _accessorCall = this.xptExternalizer.accessorCall(it.getEditorGen(), 
      this._externalizerUtils_qvto.nameKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)));
    _builder.append(_accessorCall, "\t");
    _builder.append(", view);");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("referencedElementSelectionPage.setTitle(");
    CharSequence _accessorCall_1 = this.xptExternalizer.accessorCall(it.getEditorGen(), 
      this._externalizerUtils_qvto.titleKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)));
    _builder.append(_accessorCall_1, "\t");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("referencedElementSelectionPage.setDescription(");
    CharSequence _accessorCall_2 = this.xptExternalizer.accessorCall(it.getEditorGen(), 
      this._externalizerUtils_qvto.descriptionKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)));
    _builder.append(_accessorCall_2, "\t");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("referencedElementSelectionPage.setModelElement(modelElement);");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("this.editingDomain = editingDomain;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_3 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_3, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public void addPages() {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("addPage(referencedElementSelectionPage);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_4 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_4, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public boolean performFinish() {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequest.ViewDescriptor viewDescriptor =");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("new org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequest.ViewDescriptor(");
    _builder.newLine();
    _builder.append("\t\t\t\t\t\t");
    _builder.append("new org.eclipse.gmf.runtime.emf.core.util.EObjectAdapter(referencedElementSelectionPage.getModelElement()),");
    _builder.newLine();
    _builder.append("\t\t\t\t\t\t");
    _builder.append("org.eclipse.gmf.runtime.notation.Node.class, null, ");
    CharSequence _preferenceHintAccess = this.xptActivator.preferenceHintAccess(it.getEditorGen());
    _builder.append(_preferenceHintAccess, "\t\t\t\t\t\t");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("org.eclipse.gmf.runtime.common.core.command.ICommand command =");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("new org.eclipse.gmf.runtime.diagram.ui.commands.CreateCommand(");
    _builder.newLine();
    _builder.append("\t\t\t\t\t\t");
    _builder.append("editingDomain, viewDescriptor, referencedElementSelectionPage.getView());");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("command = command.compose(new ");
    CharSequence _qualifiedClassName = this.xptCreateShortcutDecorationsCommand.qualifiedClassName(it);
    _builder.append(_qualifiedClassName, "\t\t");
    _builder.append("(");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t");
    _builder.append("editingDomain, referencedElementSelectionPage.getView(), viewDescriptor));");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("try {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("org.eclipse.core.commands.operations.OperationHistoryFactory.getOperationHistory().execute(");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("command, new org.eclipse.core.runtime.NullProgressMonitor(), null);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("} catch (org.eclipse.core.commands.ExecutionException ee) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    CharSequence _qualifiedClassName_1 = this.xptActivator.qualifiedClassName(it.getEditorGen().getPlugin());
    _builder.append(_qualifiedClassName_1, "\t\t\t");
    _builder.append(".getInstance().logError(\"Unable to create shortcut\", ee); ");
    CharSequence _nonNLS = this._common.nonNLS(1);
    _builder.append(_nonNLS, "\t\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return true;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedClassComment_1 = this._common.generatedClassComment();
    _builder.append(_generatedClassComment_1, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private static class ReferencedElementSelectionPage extends ");
    CharSequence _qualifiedClassName_2 = this.xptModelElementSelectionPage.qualifiedClassName(it);
    _builder.append(_qualifiedClassName_2, "\t");
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence _generatedMemberComment_5 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_5, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("private org.eclipse.gmf.runtime.notation.View view;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence _generatedMemberComment_6 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_6, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("public ReferencedElementSelectionPage(String pageName, org.eclipse.gmf.runtime.notation.View view) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("super(pageName);");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("this.view = view;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence _generatedMemberComment_7 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_7, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("public org.eclipse.gmf.runtime.notation.View getView() {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return view;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence _generatedMemberComment_8 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_8, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("protected String getSelectionTitle() {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return ");
    CharSequence _accessorCall_3 = this.xptExternalizer.accessorCall(it.getEditorGen(), this._externalizerUtils_qvto.messageKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)));
    _builder.append(_accessorCall_3, "\t\t\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence _generatedMemberComment_9 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_9, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("protected boolean validatePage() {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("if (selectedModelElement == null) {");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("setErrorMessage(");
    CharSequence _accessorCall_4 = this.xptExternalizer.accessorCall(it.getEditorGen(), 
      this.i18nKeyForShortcutWizardReferencedElementPageEmptyError(it));
    _builder.append(_accessorCall_4, "\t\t\t\t");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t\t");
    _builder.append("return false;");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("boolean result = org.eclipse.gmf.runtime.diagram.core.services.ViewService.getInstance().provides(");
    _builder.newLine();
    _builder.append("\t\t\t\t\t");
    _builder.append("org.eclipse.gmf.runtime.notation.Node.class,");
    _builder.newLine();
    _builder.append("\t\t\t\t\t");
    _builder.append("new org.eclipse.gmf.runtime.emf.core.util.EObjectAdapter(selectedModelElement),");
    _builder.newLine();
    _builder.append("\t\t\t\t\t");
    _builder.append("view, null, org.eclipse.gmf.runtime.diagram.core.util.ViewUtil.APPEND, true,");
    _builder.newLine();
    _builder.append("\t\t\t\t\t");
    CharSequence _preferenceHintAccess_1 = this.xptActivator.preferenceHintAccess(it.getEditorGen());
    _builder.append(_preferenceHintAccess_1, "\t\t\t\t\t");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t");
    _builder.append("setErrorMessage(result ? null : ");
    CharSequence _accessorCall_5 = this.xptExternalizer.accessorCall(it.getEditorGen(), 
      this.i18nKeyForShortcutWizardReferencedElementPageInvalidError(it));
    _builder.append(_accessorCall_5, "\t\t\t");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t\t");
    _builder.append("return result;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  @Localization
  public CharSequence i18nValues(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      GenApplication _application = it.getEditorGen().getApplication();
      boolean _notEquals = (!Objects.equal(null, _application));
      if (_notEquals) {
        CharSequence _messageEntry = this.xptExternalizer.messageEntry(this._externalizerUtils_qvto.nameKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)), "Select referenced element");
        _builder.append(_messageEntry);
        _builder.newLineIfNotEmpty();
        CharSequence _messageEntry_1 = this.xptExternalizer.messageEntry(this._externalizerUtils_qvto.titleKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)), "Referenced element");
        _builder.append(_messageEntry_1);
        _builder.newLineIfNotEmpty();
        CharSequence _messageEntry_2 = this.xptExternalizer.messageEntry(this._externalizerUtils_qvto.descriptionKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)), 
          "Select element that the new shortcut will refer to.");
        _builder.append(_messageEntry_2);
        _builder.newLineIfNotEmpty();
        CharSequence _messageEntry_3 = this.xptExternalizer.messageEntry(this._externalizerUtils_qvto.messageKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)), 
          "Select referenced element:");
        _builder.append(_messageEntry_3);
        _builder.newLineIfNotEmpty();
        CharSequence _messageEntry_4 = this.xptExternalizer.messageEntry(this.i18nKeyForShortcutWizardReferencedElementPageEmptyError(it), 
          "Referenced element is not selected");
        _builder.append(_messageEntry_4);
        _builder.newLineIfNotEmpty();
        CharSequence _messageEntry_5 = this.xptExternalizer.messageEntry(this.i18nKeyForShortcutWizardReferencedElementPageInvalidError(it), 
          "Invalid referenced element is selected");
        _builder.append(_messageEntry_5);
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  @Localization
  public CharSequence i18nAccessors(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      GenApplication _application = it.getEditorGen().getApplication();
      boolean _notEquals = (!Objects.equal(null, _application));
      if (_notEquals) {
        CharSequence _accessorField = this.xptExternalizer.accessorField(this._externalizerUtils_qvto.nameKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)));
        _builder.append(_accessorField);
        _builder.newLineIfNotEmpty();
        CharSequence _accessorField_1 = this.xptExternalizer.accessorField(this._externalizerUtils_qvto.titleKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)));
        _builder.append(_accessorField_1);
        _builder.newLineIfNotEmpty();
        CharSequence _accessorField_2 = this.xptExternalizer.accessorField(this._externalizerUtils_qvto.descriptionKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)));
        _builder.append(_accessorField_2);
        _builder.newLineIfNotEmpty();
        CharSequence _accessorField_3 = this.xptExternalizer.accessorField(this._externalizerUtils_qvto.messageKey(this.i18nKeyForShortcutWizardReferencedElementPage(it)));
        _builder.append(_accessorField_3);
        _builder.newLineIfNotEmpty();
        CharSequence _accessorField_4 = this.xptExternalizer.accessorField(this.i18nKeyForShortcutWizardReferencedElementPageEmptyError(it));
        _builder.append(_accessorField_4);
        _builder.newLineIfNotEmpty();
        CharSequence _accessorField_5 = this.xptExternalizer.accessorField(this.i18nKeyForShortcutWizardReferencedElementPageInvalidError(it));
        _builder.append(_accessorField_5);
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  @Localization
  public String i18nKeyForShortcutWizardReferencedElementPage(final GenDiagram diagram) {
    CharSequence _className = this.className(diagram);
    return (_className + ".ReferencedElementSelectionPage");
  }
  
  @Localization
  public String i18nKeyForShortcutWizardReferencedElementPageEmptyError(final GenDiagram diagram) {
    String _i18nKeyForShortcutWizardReferencedElementPage = this.i18nKeyForShortcutWizardReferencedElementPage(diagram);
    return (_i18nKeyForShortcutWizardReferencedElementPage + "EmptyError");
  }
  
  @Localization
  public String i18nKeyForShortcutWizardReferencedElementPageInvalidError(final GenDiagram diagram) {
    String _i18nKeyForShortcutWizardReferencedElementPage = this.i18nKeyForShortcutWizardReferencedElementPage(diagram);
    return (_i18nKeyForShortcutWizardReferencedElementPage + "InvalidError");
  }
}
