/**
 * Copyright (c) 2013, 2020 Borland Software Corporation, CEA LIST, Artal and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Michael Golubev (Montages) - initial API and implementation
 *   Aurelien Didier (ARTAL) - aurelien.didier51@gmail.com - Bug 569174
 */
package xpt.plugin;

import com.google.common.base.Objects;
import com.google.inject.Inject;
import com.google.inject.Singleton;
import org.eclipse.emf.common.util.EList;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenApplication;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenNavigator;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenPlugin;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenPropertySheet;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import xpt.Common;
import xpt.GenEditorGenerator_qvto;

@Singleton
@SuppressWarnings("all")
public class manifest {
  @Inject
  @Extension
  private GenEditorGenerator_qvto _genEditorGenerator_qvto;
  
  @Inject
  @Extension
  private Common _common;
  
  public CharSequence qualifiedClassName(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("META-INF/MANIFEST.MF");
    return _builder;
  }
  
  public CharSequence fullPath(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _qualifiedClassName = this.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    return _builder;
  }
  
  public CharSequence manifest(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _manifestVersion = this.manifestVersion(it);
    _builder.append(_manifestVersion);
    _builder.newLineIfNotEmpty();
    CharSequence _bundleManifestVersion = this.bundleManifestVersion(it);
    _builder.append(_bundleManifestVersion);
    _builder.newLineIfNotEmpty();
    CharSequence _bundleName = this.bundleName(it);
    _builder.append(_bundleName);
    _builder.newLineIfNotEmpty();
    CharSequence _symbolicName = this.symbolicName(it);
    _builder.append(_symbolicName);
    _builder.newLineIfNotEmpty();
    CharSequence _bundleVersion = this.bundleVersion(it);
    _builder.append(_bundleVersion);
    _builder.newLineIfNotEmpty();
    CharSequence _bundleClassPath = this.bundleClassPath(it);
    _builder.append(_bundleClassPath);
    _builder.newLineIfNotEmpty();
    CharSequence _bundleActivator = this.bundleActivator(it);
    _builder.append(_bundleActivator);
    _builder.newLineIfNotEmpty();
    CharSequence _bundleVendor = this.bundleVendor(it);
    _builder.append(_bundleVendor);
    _builder.newLineIfNotEmpty();
    CharSequence _bundleLocalization = this.bundleLocalization(it);
    _builder.append(_bundleLocalization);
    _builder.newLineIfNotEmpty();
    CharSequence _exportPackages = this.exportPackages(it);
    _builder.append(_exportPackages);
    _builder.newLineIfNotEmpty();
    CharSequence _requireBundle = this.requireBundle(it);
    _builder.append(_requireBundle);
    _builder.newLineIfNotEmpty();
    CharSequence _bundleActivatorPolicy = this.bundleActivatorPolicy(it);
    _builder.append(_bundleActivatorPolicy);
    _builder.newLineIfNotEmpty();
    CharSequence _executionEnvironment = this.executionEnvironment(it);
    _builder.append(_executionEnvironment);
    _builder.newLineIfNotEmpty();
    CharSequence _additions = this.additions(it);
    _builder.append(_additions);
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence manifestVersion(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Manifest-Version: 1.0");
    return _builder;
  }
  
  public CharSequence bundleManifestVersion(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Bundle-ManifestVersion: 2");
    return _builder;
  }
  
  public CharSequence bundleName(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Bundle-Name: %pluginName");
    return _builder;
  }
  
  public CharSequence symbolicName(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Bundle-SymbolicName: ");
    String _iD = it.getID();
    _builder.append(_iD);
    _builder.append("; singleton:=true");
    return _builder;
  }
  
  public CharSequence bundleVersion(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Bundle-Version: ");
    String _version = it.getVersion();
    _builder.append(_version);
    return _builder;
  }
  
  public CharSequence bundleClassPath(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Bundle-ClassPath: .");
    return _builder;
  }
  
  public CharSequence bundleActivator(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Bundle-Activator: ");
    String _activatorQualifiedClassName = it.getActivatorQualifiedClassName();
    _builder.append(_activatorQualifiedClassName);
    return _builder;
  }
  
  public CharSequence bundleVendor(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Bundle-Vendor: %providerName");
    return _builder;
  }
  
  public CharSequence bundleLocalization(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Bundle-Localization: plugin");
    return _builder;
  }
  
  public CharSequence bundleActivatorPolicy(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Bundle-ActivationPolicy: lazy");
    return _builder;
  }
  
  public CharSequence additions(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder;
  }
  
  public CharSequence exportPackages(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Export-Package: ");
    String _editPartsPackageName = it.getEditorGen().getDiagram().getEditPartsPackageName();
    _builder.append(_editPartsPackageName);
    _builder.append(", ");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    String _packageName = it.getEditorGen().getEditor().getPackageName();
    _builder.append(_packageName, " ");
    _builder.append(",");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    String _providersPackageName = it.getEditorGen().getDiagram().getProvidersPackageName();
    _builder.append(_providersPackageName, " ");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence requireBundle(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Require-Bundle: org.eclipse.core.runtime,");
    _builder.newLine();
    {
      GenApplication _application = it.getEditorGen().getApplication();
      boolean _equals = Objects.equal(_application, null);
      if (_equals) {
        _builder.append(" org.eclipse.core.resources,");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      if ((it.getEditorGen().getDiagram().generateShortcutIcon() || ((!Objects.equal(it.getEditorGen().getNavigator(), null)) && it.getEditorGen().getNavigator().isGenerateDomainModelNavigator()))) {
        _builder.append(" org.eclipse.core.expressions,");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append(" org.eclipse.jface,");
    _builder.newLineIfNotEmpty();
    {
      GenApplication _application_1 = it.getEditorGen().getApplication();
      boolean _equals_1 = Objects.equal(_application_1, null);
      if (_equals_1) {
        _builder.append(" org.eclipse.ui.ide,");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append(" org.eclipse.ui.views,");
    _builder.newLineIfNotEmpty();
    {
      GenNavigator _navigator = it.getEditorGen().getNavigator();
      boolean _notEquals = (!Objects.equal(_navigator, null));
      if (_notEquals) {
        _builder.append(" org.eclipse.ui.navigator,");
        _builder.newLineIfNotEmpty();
        _builder.append(" ");
        _builder.append("org.eclipse.ui.navigator.resources,");
        _builder.newLine();
      }
    }
    _builder.append(" org.eclipse.emf.ecore,");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("org.eclipse.emf.ecore.xmi,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("org.eclipse.emf.edit.ui,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("org.eclipse.gmf.runtime.emf.core,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("org.eclipse.gmf.runtime.emf.commands.core,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("org.eclipse.gmf.runtime.emf.ui.properties,");
    _builder.newLine();
    _builder.append(" ");
    _builder.append("org.eclipse.gmf.runtime.diagram.ui,");
    _builder.newLine();
    {
      boolean _isPrintingEnabled = it.isPrintingEnabled();
      if (_isPrintingEnabled) {
        _builder.append(" org.eclipse.gmf.runtime.diagram.ui.printing,");
        _builder.newLineIfNotEmpty();
        _builder.append(" ");
        _builder.append("org.eclipse.gmf.runtime.diagram.ui.printing.render,");
        _builder.newLine();
      }
    }
    {
      GenPropertySheet _propertySheet = it.getEditorGen().getPropertySheet();
      boolean _notEquals_1 = (!Objects.equal(_propertySheet, null));
      if (_notEquals_1) {
        _builder.append(" org.eclipse.gmf.runtime.diagram.ui.properties,");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append(" org.eclipse.gmf.runtime.diagram.ui.providers,");
    _builder.newLineIfNotEmpty();
    {
      GenApplication _application_2 = it.getEditorGen().getApplication();
      boolean _equals_2 = Objects.equal(_application_2, null);
      if (_equals_2) {
        _builder.append(" org.eclipse.gmf.runtime.diagram.ui.providers.ide,");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append(" org.eclipse.gmf.runtime.diagram.ui.render,");
    _builder.newLineIfNotEmpty();
    _builder.append(" ");
    _builder.append("org.eclipse.gmf.runtime.diagram.ui.resources.editor,");
    _builder.newLine();
    {
      GenApplication _application_3 = it.getEditorGen().getApplication();
      boolean _equals_3 = Objects.equal(_application_3, null);
      if (_equals_3) {
        _builder.append(" org.eclipse.gmf.runtime.diagram.ui.resources.editor.ide,");
        _builder.newLineIfNotEmpty();
      }
    }
    EList<String> reqPlugins = it.getAllRequiredPlugins();
    _builder.newLineIfNotEmpty();
    {
      boolean _contains = reqPlugins.contains("org.eclipse.gmf.tooling.runtime");
      boolean _not = (!_contains);
      if (_not) {
        boolean notUsetBooleanVar = reqPlugins.add("org.eclipse.gmf.tooling.runtime");
        _builder.newLineIfNotEmpty();
      }
    }
    {
      for(final String reqId : reqPlugins) {
        _builder.append(" ");
        _builder.append(reqId);
        _builder.append(";visibility:=reexport,");
        CharSequence _extraLineBreak = this._common.extraLineBreak();
        _builder.append(_extraLineBreak);
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append(" org.eclipse.gef;visibility:=reexport");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence executionEnvironment(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("Bundle-RequiredExecutionEnvironment: ");
    {
      int _jdkComplianceLevel = this._genEditorGenerator_qvto.jdkComplianceLevel(it.getEditorGen());
      boolean _greaterThan = (_jdkComplianceLevel > 5);
      if (_greaterThan) {
        _builder.append("JavaSE-1.6");
      } else {
        _builder.append("J2SE-1.");
        int _jdkComplianceLevel_1 = this._genEditorGenerator_qvto.jdkComplianceLevel(it.getEditorGen());
        _builder.append(_jdkComplianceLevel_1);
      }
    }
    _builder.newLineIfNotEmpty();
    return _builder;
  }
}
